@@ -44,6 +44,7 @@
 
 #include <string.h>
 #include <unistd.h>
+#include <dirent.h>
 #include "vesa.h"
 
 /* All drivers initialising the SW cursor need this */
@@ -186,7 +187,7 @@ static IsaChipsets VESAISAchipsets[] = {
 /* 
  * This contains the functions needed by the server after loading the
  * driver module.  It must be supplied, and gets added the driver list by
- * the Module Setup funtion in the dynamic case.  In the static case a
+ * the Module Setup function in the dynamic case.  In the static case a
  * reference to this is compiled in, and this requires that the name of
  * this DriverRec be an upper-case version of the driver name.
  */
@@ -353,8 +354,6 @@ VESAValidMode(SCRN_ARG_TYPE arg, DisplayModePtr p, Bool flag, int pass)
     DisplayModePtr mode;
     float v;
 
-    pVesa = VESAGetRec(pScrn);
-
     if (pass != MODECHECK_FINAL) {
 	if (!warned) {
 	    xf86DrvMsg(pScrn->scrnIndex, X_WARNING, "VESAValidMode called unexpectedly\n");
@@ -439,12 +438,40 @@ VESAInitScrn(ScrnInfoPtr pScrn)
     pScrn->FreeScreen    = VESAFreeScreen;
 }
 
+#ifdef XSERVER_LIBPCIACCESS
+#ifdef __linux__
+/*
+ * check if a file exist in directory
+ * should be equivalent to a glob ${directory}/${prefix}*
+ */
+
+static Bool
+VESAFileExistsPrefix(const char *directory, const char *prefix) {
+    DIR *dir;
+    struct dirent *entry;
+    Bool found = FALSE;
+    int len = strlen(prefix);
+    
+    dir = opendir(directory);
+    if (!dir)
+        return FALSE;
+
+    while ((entry = readdir(dir)) != NULL) {
+        if (strlen(entry->d_name) > len && 
+            !memcmp(entry->d_name, prefix, len)) {
+            found = TRUE;
+            break;
+        }
+    }
+    closedir(dir);
+    return found;
+}
+#endif
+
 /*
  * This function is called once, at the start of the first server generation to
  * do a minimal probe for supported hardware.
  */
-
-#ifdef XSERVER_LIBPCIACCESS
 static Bool
 VESAPciProbe(DriverPtr drv, int entity_num, struct pci_device *dev,
 	     intptr_t match_data)
@@ -452,9 +479,9 @@ VESAPciProbe(DriverPtr drv, int entity_num, struct pci_device *dev,
     ScrnInfoPtr pScrn;
 
 #ifdef __linux__
-    if (access("/sys/devices/platform/efi-framebuffer.0", F_OK) == 0 ||
-        access("/sys/devices/platform/efifb.0", F_OK) == 0) {
-        ErrorF("vesa: Refusing to run on UEFI\n");
+    if (VESAFileExistsPrefix("/dev", "fb") || 
+        VESAFileExistsPrefix("/dev/dri", "card")) {
+        ErrorF("vesa: Refusing to run, Framebuffer or dri device present\n");
         return FALSE;
     }
 #endif
@@ -1787,7 +1814,6 @@ static Bool
 VESADGASetMode(ScrnInfoPtr pScrn, DGAModePtr pDGAMode)
 {
     DisplayModePtr pMode;
-    int scrnIdx = pScrn->pScreen->myNum;
     int frameX0, frameY0;
 
     if (pDGAMode) {
